#include "RPi_Pico_TimerInterrupt.h"
#include "hardware/pwm.h"
#include "sin_1024.h"
#include "185_shanai_chime.h"
#include "185_CP_C2000.h"
#include "185_door_open2.h"
#include "185_shinagawa_B.h"
#include "CrossingCountry.h"
#include "Station_Announce01.h"

//#define TIMER1_INTERVAL_us 22
#define TIMER1_INTERVAL_us 125
#define PIN_BUZZER    A0

#define C4 261
#define C+4 277
#define D4 293
#define D+4 311
#define E4 329
#define F4 349
#define F+4 369
#define G4 391
#define G+4 415
#define A4 440
#define A+4 466
#define B4 493

#define WAVE_LENGTH_A K185_shanai_chime_length 
#define WAVE_DATA_A K185_shanai_chime_data
#define WAVE_LENGTH_B K185_CP_C2000_length 
#define WAVE_DATA_B K185_CP_C2000_data
#define WAVE_LENGTH_C K185_door_open2_length 
#define WAVE_DATA_C K185_door_open2_data
#define WAVE_LENGTH_D K185_shinagawa_B_length 
#define WAVE_DATA_D K185_shinagawa_B_data
#define WAVE_LENGTH_E CrossingCountry_length 
#define WAVE_DATA_E CrossingCountry_data
#define WAVE_LENGTH_F Station_Announce01_length 
#define WAVE_DATA_F Station_Announce01_data

#define WAVE_CHORDS 6

// Init RPI_PICO_Timer
RPI_PICO_Timer ITimer1(1);

volatile uint32_t wave_val[WAVE_CHORDS];
volatile uint32_t wave_length[WAVE_CHORDS];
volatile uint32_t wave_add[WAVE_CHORDS];
volatile uint slice_num = 0;
volatile uint32_t current_val = 0;
//volatile uint8_t temp_wave[WAVE_LENGTH_A];

String inputString = "";         // a String to hold incoming data
bool stringComplete = false;  // whether the string is complete

bool TimerHandler1(struct repeating_timer *t) {
  current_val = 0;
  for(int i = 0;i <WAVE_CHORDS;i++) {
    wave_val[i] += wave_add[i];
    /*
    //when repeat
    if(wave_val[i] >= wave_length[i]) {
      wave_val[i] -= wave_length[i];
    }
    */
    //when one time
    if(wave_val[i] >= wave_length[i]) {
      wave_val[i] = 0;
      wave_add[i] = 0;
    }
    //current_val += sin_1024_data[  wave_val[i] >> 4];
    //current_val += temp_wave[  wave_val[i] >> 4];
    switch(i){
      case 0:
        current_val += WAVE_DATA_A[  wave_val[i] >> 4];
        break;
      case 1:
        current_val += WAVE_DATA_B[  wave_val[i] >> 4];
        break;
      case 2:
        current_val += WAVE_DATA_C[  wave_val[i] >> 4];
        break;
      case 3:
        current_val += WAVE_DATA_D[  wave_val[i] >> 4];
        break;
      case 4:
        current_val += WAVE_DATA_E[  wave_val[i] >> 4];
        break;
      case 5:
        current_val += WAVE_DATA_F[  wave_val[i] >> 4];
        break;
      default:
        current_val += WAVE_DATA_A[  wave_val[i] >> 4];
        break;
    }
  }
  current_val = current_val * 4 / WAVE_CHORDS;
  
  pwm_set_chan_level(slice_num, PWM_CHAN_A, current_val);
  //pwm_set_chan_level(slice_num, PWM_CHAN_B, val);  
  return true;
}

void play_wav(int8_t ch,int16_t freq) {
  wave_val[ch] = 0;
  wave_add[ch] = freq;
}
void play_cls() {
  for(int i = 0;i < WAVE_CHORDS;i++){
      wave_add[i] = 0;
      wave_val[i] = 0;
  }
}

void setup() {
  // put your setup code here, to run once:
  ITimer1.attachInterruptInterval(TIMER1_INTERVAL_us, TimerHandler1);
  gpio_set_function(PIN_BUZZER, GPIO_FUNC_PWM);
  
  slice_num = pwm_gpio_to_slice_num(PIN_BUZZER);
  pwm_set_wrap(slice_num, 1023);
  pwm_set_chan_level(slice_num, PWM_CHAN_A, 0);
  //pwm_set_chan_level(slice_num, PWM_CHAN_B, 0);
  for(int i = 0;i < WAVE_CHORDS;i++) {
    wave_val[i] = 0;
    wave_length[i] = WAVE_LENGTH_A << 4;
    wave_add[i] =0;  
  }
  wave_length[0] = WAVE_LENGTH_A << 4;
  wave_length[1] = WAVE_LENGTH_B << 4;
  wave_length[2] = WAVE_LENGTH_C << 4;
  wave_length[3] = WAVE_LENGTH_D << 4;
  wave_length[4] = WAVE_LENGTH_E << 4;
  wave_length[5] = WAVE_LENGTH_F << 4;
  /*
  for(int i = 0;i < WAVE_LENGTH_A / 2;i++) {
    temp_wave[i] = 0; 
  }
  for(int i = WAVE_LENGTH_A /2;i < WAVE_LENGTH_A;i++) {
    temp_wave[i] = 255; 
  }
  for(int i = 0;i < WAVE_LENGTH_A;i++) {
    temp_wave[i] = WAVE_DATA_A[i]; 
  }
  */
  Serial.begin(115200);
  inputString.reserve(200);
      
  pwm_set_enabled(slice_num, true);

  //play_wav(0,16);
  //play_wav(1,16);
  //play_wav(2,16);
  //play_wav(3,16);
  //play_wav(4,16);
  //play_wav(5,16);

}

void loop() {
  // put your main code here, to run repeatedly:
  if (stringComplete) {
    Serial.println(inputString);
    if(inputString.equals("0")){play_wav(0,16);}
    else if(inputString.equals("1")){play_wav(1,16);}
    else if(inputString.equals("2")){play_wav(2,16);}
    else if(inputString.equals("3")){play_wav(3,16);}
    else if(inputString.equals("4")){play_wav(4,16);}
    else if(inputString.equals("5")){play_wav(5,16);}
    else if(inputString.equals("c")){play_cls();}
    // clear the string:
    inputString = "";
    stringComplete = false;
  }  
}


void serialEvent() {
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read();
    if (inChar == '\n') {
      stringComplete = true;
      return;
    }
    // add it to the inputString:
    inputString += inChar;
  }
}
